/* cycloid.c
 * 
 * Licensed under WTFPL
 * (c) 2010 LexsZero <lexszero@gmail.com>
 *
 * Draw circle modulated with sum of some waveforms (sin, saw, square)
 * 
 * Syntax:
 * 	cycloid <base frequency> [waveform1] [waveform2] ...
 * 	waveform: <sin|saw|square> <freqency> <amplitude>
 *
 * 	Amplitude in range of int16_t
 *
 * Example:
 *  cycloid 100 sin 303 10000 sin 201 20000 sin 1 200 | aplay -c2 -f U16 -r 20000 -
 *
 * Please submit your fancy oscilloscope demos to me :)
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <inttypes.h>

typedef struct {
	enum {MOD_SIN, MOD_SAW, MOD_SQUARE} waveform;
	int i, a;
	float t;
} modulation_t;

int main(int argc, char **argv) {
	if (argc < 5) {
		fprintf(stderr, "Usage: cycloid <carrier_freq> <sin|saw|square> <modulation_freq> <amplitude>\n");
		return 0;
	}
	
	float t_car = 48000/atoi(argv[1]);
	int i_car = 0, n_mod = (argc-2)/3;

	modulation_t *mod = malloc(n_mod * sizeof(modulation_t));

	int i;
	for (i = 0; i < n_mod; i++) {
		if (strcmp(argv[i*3+2], "sin") == 0)
			mod[i].waveform = MOD_SIN;
		if (strcmp(argv[i*3+2], "saw") == 0)
			mod[i].waveform = MOD_SAW;
		if (strcmp(argv[i*3+2], "square") == 0)
			mod[i].waveform = MOD_SQUARE;

		mod[i].t = 48000/atoi(argv[i*3+3]);
		mod[i].a = atoi(argv[i*3+4]);
		mod[i].i = 0;

		printf("%i %f %i\n", mod[i].waveform, mod[i].t, mod[i].a);
	}

//	exit(0);
	uint16_t tx, ty, d;
	while (1) {
		d = 0;
		for (i = 0; i < n_mod; i++) {
			switch (mod[i].waveform) {
				case MOD_SIN: ;
					d += (sin(mod[i].i/mod[i].t*2*M_PI)+1)*mod[i].a;
					break;
				case MOD_SAW: ;
					d += mod[i].i/mod[i].t*mod[i].a;
					break;
				case MOD_SQUARE: ;
					d += (mod[i].i > mod[i].t/2) ? mod[i].a : 0;
					break;
			}
			mod[i].i = (mod[i].i+1) % (int) mod[i].t;
		}

		tx = (sin(i_car/t_car*2*M_PI)+1)*(32767-d)+d;
		ty = (cos(i_car/t_car*2*M_PI)+1)*(32767-d)+d;
		write(1, &tx, sizeof(uint16_t));
		write(1, &ty, sizeof(uint16_t));
		i_car = (i_car+1) % (int)t_car;
	}
	return 0;
}
